//+------------------------------------------------------------------+
//|                                                 SOMPredictor.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, TheXpert"
#property link      "theforexpert@gmail.com"

extern int        Magic          = 23479269;
extern int        TimesToRepeat  = 3;
extern int        Slippage       = 3;

extern double     RiskPercentage = 0;
extern double     TradeLot       = 0;

extern int        SL             = 1000;
extern int        TP             = 0;

extern string     ResultsFileName= "SOMPatternResults.csv";
extern string     NetFileName    = "SOMPattern.ennf";
extern string     ErrorNetFileName= "SOMError.ennf";
extern int        LearnMonths    = 30;

extern int        PatternSize    = 40;
extern int        FutureSize     = 5;
extern int        Threshold      = 100;
extern int        SOMSize        = 8;

extern double     LearnKoef      = 0.9;
extern double     LearnRadius    = 4;
extern int        LearnIterations= 2000;

extern bool       AutoRadiusCount= true;

extern bool       TestRun        = false;
extern bool       AutoLearn      = false;

#include <SOMNet.mqh>
#include <DragTrade_v_1.0/DragTrade_Trading.mqh>

datetime LastTime;
datetime LastMonth;
bool FirstRun;
string _Symbol;

datetime StartTime;

int init()
{
   LastTime = 0;
   LastMonth = -1;
   FirstRun = true;
   _Symbol = Symbol();
   
   SetLogName(ResultsFileName);
   
   if (AutoRadiusCount)
   {
      LearnRadius = SOMSize*0.8;
      Print("LearnRadius = ", LearnRadius);
   }
   
   if (AutoLearn)
   {
      FileDelete(ResultsFileName);
      FileDelete(NetFileName);
   }

   return(0);
}

int start()
{
   if (FirstRun)
   {
      OnFirstRun();
   }
   
   if (LastTime != Time[0])
   {
      LastTime = Time[0];
      OnNewBar();
   }
   else return (0);
   
   datetime startMonth = TimeMonth(TimeCurrent());
   if (LastMonth != startMonth)
   {
      //new month
      OnNewMonth();
      LastMonth = startMonth;
   }
   
   FirstRun = false;

   return(0);
}

int deinit()
{
   if (TestRun) LearnPatterns();
}

void OnFirstRun()
{
   StartTime = TimeCurrent();
   ClearLog();
}

void OnNewBar()
{
   if (AutoLearn || TestRun) PreparePattern();
   
   if (!TestRun)
   {
      if (AutoLearn)
      {
         if (TimeCurrent() - StartTime < LearnMonths*30*24*60*60) return;
      }
      
      double input[];
      
      if (!PrepareInput(1, input)) return;
      int inSize = ArraySize(input);

      double DynamicID = GetTickCount();
      while(!RegisterUsage(DynamicID, inSize, SOMSize))
      {
         DynamicID = GetTickCount();
      }
   
      bool result = true;
      double positive, negative;
      int count;

      if (
            LoadNetFromFile(DynamicID, GetFullPath(NetFileName))
         )
      {
         double info[3];
         result = result && PropagatePattern(DynamicID, inSize, input, info);
         positive = info[0]; 
         negative = info[1];
         count = info[2];
      }
      else
      {
         Print("Load failed from ", GetFullPath(NetFileName));
         result = false;
      }
   
      UnregisterUsage(DynamicID);
      
      if (result) EnterMarket(positive, negative, count);
   }
}

void EnterMarket(double positive, double negative, int count)
{
   if (positive/(negative + 1.0) > 1.5 && count > 3)
   {
      OpenBuy(Magic, TP, SL, TradeLot);
   }
   
   if (negative/(positive + 1.0) > 1.5 && count > 3)
   {
      OpenSell(Magic, TP, SL, TradeLot);
   }
   
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != Magic && Magic != -1) continue;
      
      if(OrderOpenTime() < TimeCurrent() - FutureSize*Period()*60)
      {
         OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage);
      }
   }
}

void OnNewMonth()
{
   if (!TestRun && AutoLearn) LearnPatterns();
}

void LearnPatterns()
{
   Randomize(GetTickCount());
   if (AutoLearn)
   {
      if (TimeCurrent() - StartTime < LearnMonths*30*24*60*60) return;
   }

   Print("LearnPatterns");
   int hFile = FileOpen(ResultsFileName, FILE_CSV | FILE_READ, " ");
   if (hFile == -1) return;
   
   Print("File opened");

   double input[];
   datetime date;
   
   SetLogName("Check" + ResultsFileName);
   
   double tempOut;
   if (!ReadPattern(hFile, date, input, tempOut))
   {
      FileClose(hFile);
      Print("Pattern not read");
      return;
   }

   double output[1];
   output[0] = tempOut;
   
   int inSize = ArraySize(input);
   
   double DynamicID = GetTickCount();
   while(!RegisterUsage(DynamicID, inSize, SOMSize))
   {
      DynamicID = GetTickCount();
   }
   
   int count = 0;
   if (Time[0] < date + LearnMonths*60*60*24*30)
   {
      AddPatternsToNet(DynamicID, 1, inSize, input, output);
      count++;
   }

   while (ReadPattern(hFile, date, input, tempOut))
   {
      output[0] = tempOut;
      if (Time[0] < date + LearnMonths*60*60*24*30)
      {
         AddPatternsToNet(DynamicID, 1, inSize, input, output);
         count++;
      }
   }

   SetLogName(ResultsFileName);
   
   if (count > 50)
   {
      int start = GetTickCount();
      Print("Starting Learn, count = ", count);

      ConfigureNetLearn(DynamicID, LearnKoef, LearnRadius, LearnIterations);
      if (LearnNet(DynamicID))
      {
         SaveNetToFile(DynamicID, GetFullPath(NetFileName));
      }
      else
      {
         FileDelete(NetFileName);
      }
      
      Print("Learn lasted ", (GetTickCount() - start)/1000, " seconds");
      Print("Should have been saved to -- ", GetFullPath(NetFileName));
   }
   else
   {
      Print("Not enough data, count = ", count);
   }
   
   UnregisterUsage(DynamicID);   

   FileClose(hFile);
}

bool PrepareInput(int index, double& input[])
{
   if (Bars < PatternSize) return (false);
   
   double min = Low[index];
   double max = High[index];

   for (int i = index + 1; i < index + PatternSize; i++)
   {
      if (min > Low[i]) min = Low[i];
      if (max < High[i]) max = High[i];
   }

   ArrayResize(input, PatternSize);
   for (i = 0; i < PatternSize; i++)
   {
      input[i] = (Close[index + i] - min)/(max - min);
   }
   
   return (true);
}

bool PrepareOutput(int index, double& output)
{
   if (index <= FutureSize) return (false);
   
   double start = Close[index];
   double end = Close[index - FutureSize];

   output = 0;
   if (end > start + Threshold*Point) output = 1;
   if (end < start - Threshold*Point) output = -1;
   
   return (true);
}

void PreparePattern()
{
   double input[];
   double output[1];
   
   if (!PrepareInput(FutureSize + 1, input)) return;
   double tempOut;
   if (!PrepareOutput(FutureSize + 1, tempOut)) return;
   output[0] = tempOut;
   
   Log(Time[FutureSize + 1]);
   LogArray(input);
   LogArray(output);
}

bool ReadPattern(int hFile, datetime& date, double& input[], double& output)
{
   if (FileIsEnding(hFile)) return (false);
   
   date = FileReadNumber(hFile);
   if (date == 0) date = FileReadNumber(hFile);
   
   int inSize = FileReadNumber(hFile);

   if (inSize == 0) inSize = FileReadNumber(hFile);
   ArrayResize(input, inSize);
   
   for (int i = 0; i < inSize; i++)
   {
      input[i] = FileReadNumber(hFile);
   }
   
   int outSize = FileReadNumber(hFile);
   if (outSize != 1) outSize = FileReadNumber(hFile);
   if (outSize != 1) return (false);

   output = FileReadNumber(hFile);
   
   if (inSize <= 0) return (false);
   
   return (true);
}

//********************************************************

string LogFileName;

void SetLogName(string fileName)
{
   LogFileName = fileName;
}

void LogArray(double array[])
{
   int size = ArraySize(array);
   string s = size + " ";
   for (int i = 0; i < size; i++)
   {
      s = s + array[i] + " ";
   }
   Log(s);
}

void Log(string s)
{
   int hFile = FileOpen(LogFileName, FILE_READ | FILE_WRITE | FILE_CSV, " ");
   if (hFile == -1) return;
   
   if (!FileSeek(hFile, 0, SEEK_END)) 
   {
      FileClose(hFile); 
      return;
   }
   
   FileWrite(hFile, s);
   FileClose(hFile);
}

void ClearLog()
{
   int hFile = FileOpen(ResultsFileName, FILE_WRITE | FILE_CSV, " ");
   if (hFile == -1) return;
   
   FileClose(hFile);
}

string GetFullPath(string name)
{
   static string result;
   result = TerminalPath() + "/Experts/Files/" + name;
   if (IsTesting()) result = TerminalPath() + "/Tester/Files/" + name;
   
   return (result);
}